'' Propeller SumoBot.spin

'' Emulates basic Parallax SumoBot functionality.

'' Click the Propeller SumoBot Schematic folder in the object browser to view the
'' schematic.  

CON

  _xinfreq = 5_000_000                      
  _clkmode = xtal1 | pll16x                              

  
  L = 1                                      ' Left and right aray elements
  R = 0

VAR
 
  long qti                                   ' Stores Qtis object results

OBJ

  schematic : "Propeller SumoBot Schematic"
  piezo     : "Piezospeaker"
  ir[2]     : "Ir Detector" 
  bot       : "Dual Servo Robot Controller with Ramping"
  qtis      : "Qtis"

PUB init

  piezo.beep(6, 2637, 2637 * 2 )            ' Beep to anounce program start
  dira[24..27]~~                             ' LEDs on
  outa[24..27]~
  waitcnt(clkfreq * 2 + cnt)                 ' PAUSE 5 s
  outa[24..27]~~                             ' LEDs off
  ir[L].init(8, 9)                           ' Left IR LED + receiver
  ir[R].init(3, 2)                           ' Right IR LED + receiver
  qtis.start(14, 15,@qti, 1, clkfreq/2000)   ' Start qti cog-object
  bot.start(0, 1, bot#SIT_STILL, 0, 0)       ' Start the Dual Servo... cog-object
   
  main                                       ' Call the main method

PUB main | obstacle[2], temp                 ' 2-element local variable

  ' The two sets of sensors monitored in the repeat loop below depend on objects
  ' that are designed using different approaches.  One thing it demonstrates is
  ' that there just isn't "one way" a sensor object should be designed.  

  ' The Ir Detector objects do their work in the same cog, and they only emit
  ' 38 kHz IR and check the IR receivers when their Object method is called.
  ' On the other hand, the Qtis  object is running in another cog, repeatedly
  ' checking the qtis and updating the value of this object's qti variable.  
  repeat           

    ' The IR Detector object has methods you can call whenever you want to know
    ' if a given IR detector sees an object in its path.
    obstacle[L] := outa[24] := ir[L].Object              
    obstacle[R] := outa[27] := ir[R].Object

    '  Choose maneuver and update bot's maneuver.
    if obstacle[L] == 0 and obstacle[R] == 0   
      bot.go(bot#FORWARD)
    elseif obstacle[L] == 0
      bot.go(bot#ROTATE_LEFT)
    elseif obstacle[R] == 0
      bot.go(bot#ROTATE_RIGHT)
    else
      bot.go(bot#FORWARD)

   ' Since the Qtis object is rpeatedly checking QTI modules in another cog and
   ' updating the value of this object's qti variable, all the code below has to
   ' do is choose the maneuver based on the the qti variable's current value.  
    temp := qti
    outa[26..25] := (temp & %11)
    case temp
      %00, %01, %10: bot.go(bot#BACKWARD)
                     waitcnt(clkfreq/2 + cnt)
    case temp
      %00, %10:      bot.go(bot#ROTATE_RIGHT)
      %01:           bot.go(bot#ROTATE_LEFT)
    case temp
      %00, %01, %10: waitcnt(clkfreq*2/3 + cnt)        