{{File: Qtis.spin

Demonstrates a way of reading QTI sensors repeatedly in a separate cog and
updating a variable in the parent cog during each update.

}}

OBJ

    rc        : "Rc Decay"

VAR

  ' Order matters for declarations of sPin, ePin, and address
  ' because of longmove in start method.
  long sPin, ePin, address, qtiBits  
  long stack[30], cog
  long time

PUB start(_sPin, _ePin, _address, _mode, _time) : okay 

  {{ Starts the servos cog.
     _sPin and _ePin are the endpoints of the contiguous group of QTI signal pins.
     _address is the address of the variable that receives the QTI bit values.
     _mode 0 Sets the QTI discharge time threshold to _time
           1 Sets the QTI discharge time by sampling a black surface
     _time QTI discharge time when mode is 0 
     Returns t/f for success.
  }}
  
  longmove(@sPin, @_sPin, 3)

  if _mode == 1
    CalibrateBlack
  else
    time := _time
    
  okay := cog := cognew(qtis, @stack) + 1

PUB stop

  '' Stop servos object and free the cog

  if cog
    cogstop(cog~ - 1)

PUB qtis

{{This method repeatedly measures the surface monitored by one or more QTI sensors
and stores the result in the parent object's variable (determined by _address in
the start method).}}

  if time == 0
    time := clkfreq/2000

  outa[ePin..sPin]~~

  repeat

    dira[sPin..ePin]~~
    waitcnt(time + cnt)
    dira[sPin..ePin]~
    waitcnt(time + cnt)
    qtiBits := ina[sPin..ePin]
    long[address] := qtiBits
     
PRI CalibrateBlack | pin, temp 

  repeat pin from sPin to ePin
    time += rc.decay(pin)
  time /= (|| (ePin - sPin)) + 1
  time /= 4
  time #>= 381    